#include <WiFiEsp.h>
#include <SoftwareSerial.h>

#define WIFI_SSID "__NOME_DELLA_RETE__"
#define WIFI_PASSWORD "__PASSWORD__"
#define RED_PIN 11
#define GREEN_PIN 10
#define BLUE_PIN 9

SoftwareSerial esp(2, 3); // rx, tx
WiFiEspServer server(80);
String color = "off";

void setup() {
  Serial.begin(9600);
  esp.begin(9600);
  WiFi.init(&esp);

  pinMode(RED_PIN, OUTPUT);
  pinMode(GREEN_PIN, OUTPUT);
  pinMode(BLUE_PIN, OUTPUT);
  
  while (WiFi.begin(WIFI_SSID, WIFI_PASSWORD) != WL_CONNECTED);
  
  Serial.println();
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());
  Serial.println();
  
  server.begin();
}

void loop() {
  WiFiEspClient client = server.available();  
  if (client) {
    String command = getRequestUrl(client);
    if (command == "/red") {
      rgb(255, 0, 0);
      color = "red";
      sendResponseHeader(client);
      client.print("{\"status\": \"ok\"}");
    } else if (command == "/green") {
      rgb(0, 255, 0);
      color = "green";
      sendResponseHeader(client);
      client.print("{\"status\": \"ok\"}");
    } else if (command == "/blue") {
      rgb(0, 0, 255);
      color = "blue";
      sendResponseHeader(client);
      client.print("{\"status\": \"ok\"}");
    } else if (command == "/off") {
      rgb(0, 0, 0);
      color = "off";
      sendResponseHeader(client);
      client.print("{\"status\": \"ok\"}");
    } else if (command == "/") {
      sendResponseHeader(client);
      client.print("{\"status\": \"ok\", ");
      client.print("\"color\": \"");
      client.print(color);
      client.print("\"}");
    } else {
      sendResponseHeader(client);
      client.print("{\"status\": \"error\", ");
      client.print("\"message\": \"Comando non riconosciuto\"}");
    }      
    
    Serial.println(command);
    delay(10);
    client.stop();
  }
}

void rgb(int r, int g, int b){
  analogWrite(RED_PIN, r);
  analogWrite(GREEN_PIN, g);
  analogWrite(BLUE_PIN, b);
}

String getRequestUrl(WiFiEspClient client) {
  RingBuffer buffer(8);
  String url = "";
  bool parseUrl = false;
  while (client.connected()) {
    if (client.available()) {
      char c = client.read();
      buffer.push(c);
 
      if (buffer.endsWith("\r\n\r\n")) {
        break;
      }
      
      if (parseUrl) {
        url.concat(c);
      }     

      if (buffer.endsWith("GET ")) {
        parseUrl = true;
      }
    
      if (parseUrl && buffer.endsWith(" HTTP/")) {
        parseUrl = false;
        url.remove(url.length()-6);
        client.flush();
        break;
      }
    }
  }
  return url;
}

void sendResponseHeader(WiFiEspClient client) {
  client.println("HTTP/1.1 200 OK");
  client.println("Content-type:application/json");
  client.println("Access-Control-Allow-Origin: *");
  client.println();
}
